@@ -33,27 +33,23 @@ use dmntk_model::model::{Definitions, ItemDefinition, ItemDefinitionType, NamedE
 use std::collections::{BTreeMap, HashMap};
 
 /// Type of closure that evaluates the item definition context.
-type EvaluatorFn = Box<dyn Fn(&Name, &mut FeelContext, &ItemDefinitionContextEvaluator) -> FeelType>;
+type ItemDefinitionContextEvaluatorFn = Box<dyn Fn(&Name, &mut FeelContext, &ItemDefinitionContextEvaluator) -> FeelType>;
 
 /// Item definition type evaluators.
 #[derive(Default)]
 pub struct ItemDefinitionContextEvaluator {
-  /// Map of item definition context evaluators.
-  /// Keys are type reference names that uniquely identify item definitions.
-  /// Values are evaluation closures.
-  evaluators: HashMap<String, EvaluatorFn>,
+  evaluators: HashMap<String, ItemDefinitionContextEvaluatorFn>,
 }
 
 impl ItemDefinitionContextEvaluator {
   /// Creates item definition type evaluators.
-  pub fn new(definitions: &Definitions) -> Result<Self> {
-    let mut evaluators = HashMap::new();
+  pub fn build(&mut self, definitions: &Definitions) -> Result<()> {
     for item_definition in definitions.item_definitions() {
       let evaluator = item_definition_context_evaluator(item_definition)?;
       let type_ref = item_definition.name().to_string();
-      evaluators.insert(type_ref, evaluator);
+      self.evaluators.insert(type_ref, evaluator);
     }
-    Ok(Self { evaluators })
+    Ok(())
   }
   /// Evaluates a context from item definition with specified type reference name.
   pub fn eval(&self, type_ref: &str, name: &Name, ctx: &mut FeelContext) -> FeelType {
@@ -66,7 +62,7 @@ impl ItemDefinitionContextEvaluator {
 }
 
 ///
-fn item_definition_context_evaluator(item_definition: &ItemDefinition) -> Result<EvaluatorFn> {
+fn item_definition_context_evaluator(item_definition: &ItemDefinition) -> Result<ItemDefinitionContextEvaluatorFn> {
   match super::item_definition_type(item_definition)? {
     ItemDefinitionType::SimpleType(feel_type) => simple_type_context_evaluator(feel_type),
     ItemDefinitionType::ReferencedType(ref_type) => referenced_type_context_evaluator(ref_type),
@@ -78,7 +74,7 @@ fn item_definition_context_evaluator(item_definition: &ItemDefinition) -> Result
 }
 
 ///
-fn simple_type_context_evaluator(feel_type: FeelType) -> Result<EvaluatorFn> {
+fn simple_type_context_evaluator(feel_type: FeelType) -> Result<ItemDefinitionContextEvaluatorFn> {
   if matches!(
     feel_type,
     FeelType::String
@@ -100,18 +96,18 @@ fn simple_type_context_evaluator(feel_type: FeelType) -> Result<EvaluatorFn> {
 }
 
 ///
-fn referenced_type_context_evaluator(ref_type: String) -> Result<EvaluatorFn> {
+fn referenced_type_context_evaluator(ref_type: String) -> Result<ItemDefinitionContextEvaluatorFn> {
   Ok(Box::new(
     move |name: &Name, ctx: &mut FeelContext, evaluator: &ItemDefinitionContextEvaluator| evaluator.eval(&ref_type, name, ctx),
   ))
 }
 
 ///
-fn component_type_context_evaluator(item_definition: &ItemDefinition) -> Result<EvaluatorFn> {
-  let mut context_evaluators: Vec<(Name, EvaluatorFn)> = vec![];
+fn component_type_context_evaluator(item_definition: &ItemDefinition) -> Result<ItemDefinitionContextEvaluatorFn> {
+  let mut context_evaluators: Vec<(Name, ItemDefinitionContextEvaluatorFn)> = vec![];
   for component_item_definition in item_definition.item_components() {
     context_evaluators.push((
-      component_item_definition.name().into(),
+      component_item_definition.feel_name().as_ref().ok_or_else(err_empty_feel_name)?.clone(),
       item_definition_context_evaluator(component_item_definition)?,
     ));
   }
@@ -130,7 +126,7 @@ fn component_type_context_evaluator(item_definition: &ItemDefinition) -> Result<
 }
 
 ///
-fn collection_of_simple_type_context_evaluator(feel_type: FeelType) -> Result<EvaluatorFn> {
+fn collection_of_simple_type_context_evaluator(feel_type: FeelType) -> Result<ItemDefinitionContextEvaluatorFn> {
   if matches!(
     feel_type,
     FeelType::String
@@ -154,7 +150,7 @@ fn collection_of_simple_type_context_evaluator(feel_type: FeelType) -> Result<Ev
 }
 
 ///
-fn collection_of_referenced_type_context_evaluator(type_ref: String) -> Result<EvaluatorFn> {
+fn collection_of_referenced_type_context_evaluator(type_ref: String) -> Result<ItemDefinitionContextEvaluatorFn> {
   Ok(Box::new(
     move |name: &Name, ctx: &mut FeelContext, evaluator: &ItemDefinitionContextEvaluator| {
       let mut evaluated_ctx = FeelContext::default();
@@ -168,11 +164,11 @@ fn collection_of_referenced_type_context_evaluator(type_ref: String) -> Result<E
 }
 
 ///
-fn collection_of_component_type_context_evaluator(item_definition: &ItemDefinition) -> Result<EvaluatorFn> {
-  let mut context_evaluators: Vec<(Name, EvaluatorFn)> = vec![];
+fn collection_of_component_type_context_evaluator(item_definition: &ItemDefinition) -> Result<ItemDefinitionContextEvaluatorFn> {
+  let mut context_evaluators: Vec<(Name, ItemDefinitionContextEvaluatorFn)> = vec![];
   for component_item_definition in item_definition.item_components() {
     context_evaluators.push((
-      component_item_definition.name().into(),
+      component_item_definition.feel_name().as_ref().ok_or_else(err_empty_feel_name)?.clone(),
       item_definition_context_evaluator(component_item_definition)?,
     ));
   }
@@ -195,16 +191,21 @@ fn collection_of_component_type_context_evaluator(item_definition: &ItemDefiniti
 #[cfg(test)]
 mod tests {
   use crate::builders::item_definition_context::ItemDefinitionContextEvaluator;
-  use dmntk_common::Stringify;
   use dmntk_examples::item_definition::*;
   use dmntk_feel::context::FeelContext;
   use dmntk_feel::values::{Value, Values};
   use dmntk_feel::{FeelType, Name};
 
+  /// Utility function for building item definition evaluator from definitions.
+  fn build_evaluator(xml: &str, source: &str) -> ItemDefinitionContextEvaluator {
+    let mut evaluator = ItemDefinitionContextEvaluator::default();
+    evaluator.build(&dmntk_model::parse(xml, source).unwrap()).unwrap();
+    evaluator
+  }
+
   #[test]
   fn simple_type_string() {
-    let definitions = &dmntk_model::parse(DMN_0101, "file:///0101.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0101, "file:///0101.dmn");
     let mut ctx = FeelContext::default();
     let expected_type = FeelType::String;
     let mut expected_context = FeelContext::default();
@@ -212,13 +213,12 @@ mod tests {
     let actual_type = evaluator.eval("tCustomerName", &"Customer Name".into(), &mut ctx);
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
-    assert_eq!("{Customer Name:type(String)}", ctx.stringify());
+    assert_eq!("{Customer Name: type(string)}", ctx.to_string());
   }
 
   #[test]
   fn simple_type_number() {
-    let definitions = &dmntk_model::parse(DMN_0102, "file:///0102.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0102, "file:///0102.dmn");
     let mut ctx = FeelContext::default();
     let expected_type = FeelType::Number;
     let mut expected_context = FeelContext::default();
@@ -226,7 +226,7 @@ mod tests {
     let actual_type = evaluator.eval("tMonthlySalary", &"Monthly Salary".into(), &mut ctx);
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
-    assert_eq!("{Monthly Salary:type(Number)}", ctx.stringify());
+    assert_eq!("{Monthly Salary: type(number)}", ctx.to_string());
   }
   /*
       #[test]
@@ -274,8 +274,7 @@ mod tests {
 
   #[test]
   fn referenced_type_string() {
-    let definitions = &dmntk_model::parse(DMN_0201, "file:///0201.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0201, "file:///0201.dmn");
     let mut ctx = FeelContext::default();
     let expected_type = FeelType::String;
     let mut expected_context = FeelContext::default();
@@ -283,7 +282,7 @@ mod tests {
     let actual_type = evaluator.eval("tCustomerName", &"Customer Name".into(), &mut ctx);
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
-    assert_eq!("{Customer Name:type(String)}", ctx.stringify());
+    assert_eq!("{Customer Name: type(string)}", ctx.to_string());
   }
 
   /*
@@ -297,8 +296,7 @@ mod tests {
 
   #[test]
   fn component_type() {
-    let definitions = &dmntk_model::parse(DMN_0301, "file:///0301.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0301, "file:///0301.dmn");
     let mut ctx = FeelContext::default();
     let name_principal: Name = "principal".into();
     let name_rate: Name = "rate".into();
@@ -314,13 +312,15 @@ mod tests {
     let actual_type = evaluator.eval("tLoan", &"Loan".into(), &mut ctx);
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
-    assert_eq!("{Loan:{principal:type(Number),rate:type(Number),termMonths:type(Number)}}", ctx.stringify());
+    assert_eq!(
+      "{Loan: {principal: type(number), rate: type(number), termMonths: type(number)}}",
+      ctx.to_string()
+    );
   }
 
   #[test]
   fn collection_of_simple_type_string() {
-    let definitions = &dmntk_model::parse(DMN_0401, "file:///0401.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0401, "file:///0401.dmn");
     let mut ctx = FeelContext::default();
     let expected_type = FeelType::List(Box::new(FeelType::String));
     let mut expected_context = FeelContext::default();
@@ -328,7 +328,7 @@ mod tests {
     let actual_type = evaluator.eval("tItems", &"Items".into(), &mut ctx);
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
-    assert_eq!("{Items:[type(String)]}", ctx.stringify());
+    assert_eq!("{Items: [type(string)]}", ctx.to_string());
   }
 
   /*
@@ -384,8 +384,7 @@ mod tests {
   */
   #[test]
   fn collection_of_referenced_type_string() {
-    let definitions = &dmntk_model::parse(DMN_0501, "file:///0501.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0501, "file:///0501.dmn");
     let mut ctx = FeelContext::default();
     let expected_type = FeelType::List(Box::new(FeelType::String));
     let mut expected_context = FeelContext::default();
@@ -393,13 +392,12 @@ mod tests {
     let actual_type = evaluator.eval("tItems", &"Items".into(), &mut ctx);
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
-    assert_eq!("{Items:[type(String)]}", ctx.stringify());
+    assert_eq!("{Items: [type(string)]}", ctx.to_string());
   }
 
   #[test]
   fn collection_of_component_type() {
-    let definitions = &dmntk_model::parse(DMN_0601, "file:///0601.dmn").unwrap();
-    let evaluator = ItemDefinitionContextEvaluator::new(definitions).unwrap();
+    let evaluator = build_evaluator(DMN_0601, "file:///0601.dmn");
     let mut ctx = FeelContext::default();
     let name_manager: Name = "manager".into();
     let name_name: Name = "name".into();
@@ -418,7 +416,7 @@ mod tests {
     let mut expected_context = FeelContext::default();
     expected_context.set_entry(&"Items".into(), Value::List(Values::new(vec![Value::Context(inner_context)])));
     let actual_type = evaluator.eval("tItems", &"Items".into(), &mut ctx);
-    assert_eq!("{Items:[{manager:type(String),name:type(String),number:type(Number)}]}", ctx.stringify());
+    assert_eq!("{Items: [{manager: type(string), name: type(string), number: type(number)}]}", ctx.to_string());
     assert_eq!(expected_type, actual_type);
     assert_eq!(expected_context, ctx);
   }
