@@ -15,16 +15,20 @@
  */
 
 use crate::errors::*;
+use crate::model_evaluator::ModelEvaluator;
 use dmntk_common::Result;
 use dmntk_feel::context::FeelContext;
-use dmntk_feel::value_null;
 use dmntk_feel::values::Value;
-use dmntk_model::model::Definitions;
+use dmntk_model::model::{Definitions, DmnElement, NamedElement, RequiredVariable};
 
-/// Evaluates a business knowledge model specified by name.
-pub fn evaluate_business_knowledge_model_by_name(definitions: &Definitions, name: &str, _ctx: &FeelContext) -> Result<Value> {
-  if let Some(_decision) = definitions.business_knowledge_model_by_name(name) {
-    Ok(value_null!())
+/// Evaluates a [BusinessKnowledgeModel] defined by specified name.
+pub fn evaluate_business_knowledge_model_by_name(definitions: &Definitions, name: &str, ctx: &FeelContext) -> Result<Value> {
+  if let Some(business_knowledge_model) = definitions.business_knowledge_model_by_name(name) {
+    let id = business_knowledge_model.id().as_ref().ok_or_else(err_empty_identifier)?;
+    let output_variable_name = business_knowledge_model.variable().feel_name().as_ref().ok_or_else(err_empty_feel_name)?;
+    let model_evaluator = ModelEvaluator::new(definitions)?;
+    let result = model_evaluator.evaluate_business_knowledge_model(id, ctx, output_variable_name);
+    Ok(result)
   } else {
     Err(err_business_knowledge_model_with_name_not_found(name))
   }
