@@ -19,18 +19,40 @@ extern crate dmntk_feel;
 extern crate dmntk_feel_parser;
 extern crate dmntk_model;
 extern crate dmntk_recognizer;
+#[cfg(test)]
+#[macro_use]
+extern crate lazy_static;
 #[macro_use]
 extern crate thiserror;
 
 mod builders;
 mod errors;
-mod eval;
 mod eval_bkm;
 mod eval_dec;
 mod eval_dec_service;
-mod eval_dec_table;
+mod model_evaluator;
 
+#[cfg(test)]
+mod tests;
+
+use dmntk_common::Result;
+use dmntk_feel::values::Value;
+use dmntk_feel::Scope;
 pub use eval_bkm::evaluate_business_knowledge_model_by_name;
 pub use eval_dec::evaluate_decision_by_name;
 pub use eval_dec_service::eval_decision_service_by_name;
-pub use eval_dec_table::{evaluate_decision_table_and_context, evaluate_decision_table_and_test};
+
+/// Evaluates the decision table.
+pub fn evaluate_decision_table_from_text(_scope: &Scope, _input: &str) -> Result<Value> {
+  unimplemented!()
+}
+
+/// Evaluates a decision table against specified context.
+pub fn evaluate_decision_table_and_context(_decision_table_input: &str, _context_input: &str) -> Result<Value> {
+  unimplemented!()
+}
+
+/// Evaluates all tests associated with decision table.
+pub fn evaluate_decision_table_and_test(_input: &str, _sep: &str) -> Result<(bool, Value, Value)> {
+  unimplemented!()
+}
